from .hashicorp_base import ConnBase
import hvac
import os
import json
from outfit import construct_dict_from_dotkv

class VaultCon(ConnBase):
    """Class to construct the dict properties for the app from Consul and Vault
    """
    
    exception_key = ['host','scheme', 'port', 'path']
    exception_dict = {}
    datakey = {}

    def __init__(self, params = None, exception_dict = None):
        """Constructor inisiating all properties
        """
        ConnBase.__init__(self)

        # if exception dict is known
        if exception_dict:
            self.exception_dict = exception_dict

        # construct the consul and vault params
        vault_params = self.get_configs_dict(self._content['vault'], self.exception_key) if not params else params

        # construct the vault url
        vault_params['url'] = self.exception_dict['scheme'] +'://' + self.exception_dict['host'] + ':' + str(self.exception_dict['port'])

        # construct the vault client objects
        self.vault = hvac.Client(**vault_params)
        self.vault.kv.default_kv_version = '1'

        # get the secret information from vault then save in self.secret dict
        self.secrets = self.vault.secrets.kv.read_secret(self.exception_dict['path'])['data']
   

    def _construct_data_vault(self, key = '', info = {}):
        """construct secret configuration informations of the services from vault return config dict

        Keyword arguments:
        data -- the dict information that came from configuration.py for vault_app_configs
        key -- the string of the path that will be concated with sub key in data dict (default '')
        info -- the informations of the structures of the app configurations that will be merged with secret info (default {})
        """
        result = {}
        for k, v in self.secrets.items():
            result = construct_dict_from_dotkv(result, k.split('.'), v)
        return result

    def get_secret_kv(self):
        """run config constructor return dict all configs
        """ 
        return self._construct_data_vault()



from abc import ABC
import yaml
import os
from ..utils.logger import Logger
from ..config_loader import Outfit

class ConnBase(ABC):
    exception_dict = {}

    def __init__(self):
        '''Connection Base constructor init the file config
        '''
        self._content = Outfit.content if Outfit.content else None

    def get_configs_dict(self, source = None, exception_key = None):
            """Constructing consul/vault properties from bootstrap.py return list of configs

            it will checking if the property is in exception_key list

            Keyword arguments:
                source -- The dict of vault/consul property from config parsed yaml file 
                exception_key -- The list of special condition field in source
            
            """
            params = {}
            # iterate source items
            for k,v in source.items():
                value = str(v) if type(v) == int else v
                
                def check_envvar(val):
                    return os.environ[val] if val in os.environ else None

                # checking if the string of value is in environment variables
                temp = check_envvar(v[2:len(v)-1]) if ('${' == value[0:2]) and ('}' == value[len(value)-1]) else v
                if temp == None:
                    raise Exception('var value {0}  not found'.format(value))

                # checking if the string of value is in exception keys
                if k not in exception_key:
                    # set the real value to params[k]
                    # if temp value is digit then convert to integer
                    params[k] = temp if type(temp) == int else temp
                else:
                    # if the string of value is in exception keys
                    # assing the temp value (real value) to exception_dict for index v[1:]
                    self.exception_dict[k] =temp if type(temp) == int else temp

            return params



from .db_status import DBStatus

class HealthcheckList(object):

    @staticmethod
    def setup(healthcheck, service_list):
        HealthcheckList.__healthcheck = healthcheck
        HealthcheckList.__dbstatus = DBStatus()

        service_enum = {
            'mongoengine': lambda x, y: HealthcheckList.__dbstatus.mongo_checker(x, y),
            'postgres': lambda x, y: HealthcheckList.__dbstatus.pgsql_checker(x),
            'mysql': lambda x, y: HealthcheckList.__dbstatus.mysql_checker(x),
            'redis': lambda x, y: HealthcheckList.__dbstatus.redis_checker()
        }
        

        for key, value in service_list.items():
            check_result = service_enum[key](
                value['connection'],
                value['lib_list'] if 'lib_list' in value else None) if value != None else service_enum[key](None, None)
            if isinstance(check_result, list):
                for temp in check_result:
                    HealthcheckList.__healthcheck.add_check(temp)
            else:
                HealthcheckList.__healthcheck.add_check(check_result)
        return HealthcheckList.__healthcheck



import logging
from .utils.logger import Logger
from .utils.io import load_yaml

class Outfit(object):
    
    path = None 
    content = None
    
    @staticmethod
    def setup(path):
        Outfit.content = load_yaml(path)
        log_config = Outfit.content['logconfig'] if 'logconfig' in Outfit.content else None

        Logger.mode = 'root'

        if log_config != None:
            mode = log_config['mode'] if 'mode' in log_config else 'root'
            source_type = log_config['source_type'] if 'source_type' in log_config else None
            source_location = log_config['source_location'] if 'source_location' in log_config else None
            default_type = log_config['default_type'] if 'default_type' in log_config else None
            default_location = log_config['default_location'] if 'default_location' in log_config else None

            Logger.setup_log(mode = mode, source_type = source_type, source_location = source_location,default_type = default_type, default_location = default_location)
        



from enum import Enum

class MongoLib(Enum):
    """enumeration for mongo libs for python
    """
    MONGO_ALCHEMY = 1
    PYMONGO = 2
    MONGOENGINE = 3


class DBStatus(object):

    def mongo_checker(self, dbmongo, mongo_libs):

        def mongoengine():
            import copy
            result = True, 'mongo using mongoengine ok'
            try:
                temp_dbmongo = copy.copy(dbmongo)

                class TestConn(temp_dbmongo.Document):
                    test = temp_dbmongo.StringField()

                temp = TestConn.objects()
                result = True, 'mongo using mongoengine is working'
            except AttributeError as atr:
                result = False, 'mongo using mongoengine not found'
            return result

        def pymongo():
            import copy 
            result = True, 'mongo using pymongo ok' 
            try:
                temp_dbmongo = copy.copy(dbmongo)
                temp_dbmongo.db.test_conn.find({})
                result = True, 'mongo using pymongo is working'
            except AttributeError as atr:
                result = False, 'mongo using pymongo not found'
            return result

        list_lib = [None, pymongo, mongoengine]
        result = []
        for m in mongo_libs:
            result = list_lib[m.value - 1] if m != None else False, 'mongo driver is not found'

        return result

    def sql_checker(self, db, dbprovider):
        def sql_alchemy():
            try:
                db.engine.execute('SELECT 1')
                return True, dbprovider + ' Database is ok' 
            except :
                return False, dbprovider + ' is not working'

        return sql_alchemy

    def mysql_checker(self, db):
        return self.sql_checker(db, 'MySQL')

    def pgsql_checker(self, db):
        return self.sql_checker(db, 'Postgres')



#import conf.bootstrap as config
#import conf.datakey as datakey
from .hashicorp_base import ConnBase
import consul
import os
import json
from ..utils.io import convert_yaml
from ..utils.logger import Logger

class ConsulCon(ConnBase):
    """Class to construct the dict properties for the app from Consul and Vault
    """

    exception_key = ['path']
    exception_dict = {}
    cons = None

    def __init__(self, params = None, exception_dict = None):
        """Constructor inisiating all properties
        """
        ConnBase.__init__(self)

        # if exception dict is known
        if exception_dict:
            self.exception_dict = exception_dict
        
        # construct the consul and vault params
        consul_params = self.get_configs_dict(self._content['consul'], self.exception_key) if not params else params
        print(consul_params)

        # construct the consul
        self.cons = consul.Consul(**consul_params)

   
    def get_kv(self, type = 'json'):
        """run config constructor return dict all configs
        
        Keyword arguments :
            type -- The type of the value text format
        """

        type_enum = {
            'json' : lambda x: json.loads(x.decode('utf-8')) if x else '',
            'yaml' : lambda x: convert_yaml(x) if x else ''
        }
        temp = self.cons.kv.get(self.exception_dict['path'])[1]['Value']
        result = type_enum[type](temp) 
        return result




def construct_dict_from_dotkv(dict = {}, fields = [], value = None):
    if len(fields) <= 1:
        dict[fields[0]] = value
    else:
        dict[fields[0]] = construct_dict_from_dotkv(
            dict[fields[0]] if fields[0] in dict else {}, fields[1:], value)
        
    return dict

def construct_dotkv_from_dict(source = {}, key = '', temp_result = {}):
    for k, v in source.items():
        if type(v) == dict:
            temp_result = construct_dotkv_from_dict(v,key + '.' +k if key != '' else k, temp_result)
        else:
            temp_result[key + '.' + k if key != '' else k] = v
    
    return temp_result   

def merge_dict(dict1, dict2):
    dotkv1 = construct_dotkv_from_dict(source = dict1)
    dotkv2 = construct_dotkv_from_dict(source = dict2)
    set1 = set(dotkv1)
    set2 = set(dotkv2)
    
    union = set1 | set2
    
    union_dict = {} 
    for key in union:
        union_dict = construct_dict_from_dotkv(
            union_dict, key.split('.'), dotkv2[key] if key in set2 else dotkv1[key])
    
    return union_dict
    



import yaml
import logging.config
import logging
from .io import load_yaml 

modules = ['development', 'production']

class Logger(object):
    mode = 'development'
    loggers = []
    info = logging.getLogger(mode).info
    debug = logging.getLogger(mode).debug
    error = logging.getLogger(mode).error
    critical = logging.getLogger(mode).critical

    @staticmethod
    def load_config_from_yaml_file(source_location = None, default_type = None, default_location = None):

        try:
            log_config = load_yaml(source_location)
            logging.config.dictConfig(log_config)
            
            loggers = [name for name in logging.root.manager.loggerDict]
            return loggers
        except Exception as ex:
            return Logger.source_option(default_type, default_location)

        
    @staticmethod
    def load_config_from_consulkv(source_location = None, default_type = None, default_location = None):

        from ..hashicorp.consul_config import ConsulCon

        try:
            consul_con = ConsulCon() if consul_con == None else consul_con
            consul_con.exception_dict['path'] = source_location

            config = consul_con.get_kv('yaml')

            logging.config.dictConfig(config)

            loggers = [name for name in logging.root.manager.loggerDict]
            return loggers
        except Exception as ex:
            return Logger.source_option(default_type, default_location)
 

    @staticmethod
    def load_config_from_dict(source_location = None, default_type = None, default_location = None, config_dict = {}):
        try:
            log_config = {} 

            if config_dict == {}:
                import importlib

                temp = importlib.import_module(source_location)
                log_config = temp.logging
            else:
                log_config = config_dict

            logging.config.dictConfig(log_config)
            
            loggers = [name for name in logging.root.manager.loggerDict]
            return loggers
        except Exception as ex:
            return Logger.source_option(default_type, default_location)


    @staticmethod
    def load_config_from_json_file(source_location = None, default_type = None, default_location = None):

        try:
            import json
            with open(source_location, 'rt') as f:
                log_config = json.loads(f.read())
            logging.config.dictConfig(log_config)

            loggers = [name for name in logging.root.manager.loggerDict]
            return loggers
        except Exception as ex:
            return Logger.source_option(default_type, default_location)


    @staticmethod
    def setup_log(mode = 'development', source_type = None, 
                  source_location = None, consul_con = None, config_dict = None,
                  default_type = 'yaml_file', default_location = 'conf/logging.yaml'):
        Logger.mode = mode
        Logger.loggers = Logger.source_option(source_type, source_location, default_type, default_location)
        if Logger.mode not in Logger.loggers:
            raise Exception('modules for mode '+ Logger.mode +' is not found!!') 
     
    @staticmethod
    def source_option(source_type = None, source_location = None, default_type = None, default_location = None):
        mode_enum = {
            'yaml_file' : lambda x, y, z: Logger.load_config_from_yaml_file(x, y, z),
            'consulkv' : lambda x, y, z: Logger.load_config_from_consulkv(x, y, z),
            'dictionary' : lambda x, y, z: Logger.load_config_from_dict(x, y, z),
            'json_file' : lambda x, y, z: Logger.load_config_from_json_file(x, y, z)
        }

        return mode_enum[source_type](source_location, default_type, default_location)

 



import os
import yaml

def load_yaml(path):
    current_path = os.getcwd()
    full_path = ''

    if path[0:2] == './':
        full_path = os.path.join(current_path, path[2:])
    elif path[0:2] == '..':
        full_path = os.path.join(current_path, path)
    else:
        full_path = path

    content = {}
    # get the yaml file
    with open(full_path, 'r') as stream:
        try:
            content = yaml.safe_load(stream)
        except yaml.YAMLError as err:
            Logger.error('error load yaml file ' + str(err))

    return content

def convert_yaml(stream):
    try:
        return yaml.safe_load(stream)
    except yaml.YAMLError as err:
        Logger.error('error load yaml stream text ' + str(err)) 



from .config_loader import Outfit
from .utils.db_status import DBStatus
from .utils.healtchcheck import HealthcheckList
from .utils.collections import merge_dict
from .utils.collections import construct_dict_from_dotkv
from .utils.collections import construct_dotkv_from_dict 
from .utils.logger import Logger
from .hashicorp.consul_config import ConsulCon
from .hashicorp.vault_config import VaultCon


__version__ = '0.0.4'




import io
import re 
from setuptools import find_packages
from setuptools import setup

# parse_requirements() 
with io.open('requirements.txt') as fp:
    install_requires = fp.read()

with io.open("README.rst", "rt", encoding="utf8") as f:
    readme = f.read()

with io.open("outfit/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)


CLASSIFIERS = [
    "Development Status :: 1 - Planning",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Environment :: Plugins",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Framework :: Flask",
    "Topic :: Software Development :: Libraries :: Python Modules",
]
setup(
    name="python-outfit",
    version=version,
    author="Hendri Karisma",
    author_email="situkangsayur@gmail.com",
    maintainer="Hendri Karisma",
    maintainer_email="situkangsayur@gmail.com",
    url="https://github.com/situkangsayur/python-outfit",
    download_url="https://github.com/situkangsayur/python-outfit",
    license="MIT",
    include_package_data=True,
    description= "",
    long_description_content_type="text/x-rst",
    long_description=readme,
    platforms=["any"],
    classifiers=CLASSIFIERS,
    packages=find_packages(exclude=["tests", "tests.*"]),
    install_requires=install_requires,
    tests_require=["coverage==4.2", 
                   "hvac==0.9.5", 
                   "python-consul==1.1.0", 
                   "flask==1.1.1",
                   "py-healthcheck==1.9.0",
                   "pyyaml==5.1.1"],
    extras_require={
        "dev": [
            "coverage",
            "sphinx",
            "pallets-sphinx-themes",
            "sphinxcontrib-log-cabinet",
            "sphinx-issues",
        ],
        "docs": [
            "sphinx",
            "pallets-sphinx-themes",
            "sphinxcontrib-log-cabinet",
            "sphinx-issues",
            "readme_renderer"
        ],
    }
)
